/**
 * MessageSet.java
 *
 * A synthetic message store for demonstrating smarter tree models
 *
 * @author Dj Walker-Morgan
 */


import java.util.*;

public class MessageBox
{
	LinkedHashMap messages=new LinkedHashMap();
	
	public MessageBox()
	{
	}
	
	private int msgid=1; // Internal seed value for new messages
	
	public synchronized void generateMessage()
	{
		// We are going to genereate a new message
		Long parentid=null;
		
		if(msgid>5) // If we've generated more than 5 ids, we can start parenting
		{
			// Randomly decide wether to parent or create a new thread
			
			if((Math.random()*10)>9)
			{
				parentid=null;
			}
			else
			{
				// Get all the exisiting ids in the box
				
				ArrayList listofallkeys;
				
				synchronized(messages)
				{
					listofallkeys=new ArrayList(messages.keySet());
				}
				
				// And randomly select one to be the parent
				int j=(int)(Math.random()*listofallkeys.size());
				parentid=(Long)listofallkeys.get(j);
			}
		}
		
		// Now create the message
		
		SimpleMessage sm=new SimpleMessage(new Long(msgid++),parentid);
		
		// And add it to the messagebox
		addMessage(sm);
	}
	
	private void addMessage(SimpleMessage simplemessage)
	{	synchronized(messages)
		{
			if(simplemessage.parent!=null)
			{
				SimpleMessage pasm=getMessage(simplemessage.parent);
				
				pasm.addChild(simplemessage.id);
			}
			
			messages.put(simplemessage.id,simplemessage);
		}
		
		fireNewMessage(simplemessage.id);
	}
	
	
	public String getMessageAsString(Long value)
	{
		SimpleMessage sm=getMessage(value);
		if(sm==null) return "No longer present for "+value;
		return sm.getPayload();
	}
	
	public int getNumberOfCommentingMessages(Long id)
	{
		SimpleMessage simplemessage=getMessage(id);
		
		if(simplemessage==null) return 0;
		
		return simplemessage.children.size();
	}
	
	public Long getChildMessageId(Long id, int index)
	{
		SimpleMessage simplemessage=getMessage(id);
		
		if(simplemessage==null)	return null;
		
		return (Long)simplemessage.getChild(index);
	}
	
	
	public void deleteMessage(Long msgid)
	{
		while(getNumberOfCommentingMessages(msgid)>0)
		{
			Long child=getChildMessageId(msgid,0);
			deleteMessage(child);
		}
		
		Long parent=getParentMessageId(msgid);
		
		int i;
		
		if(parent!=null)
		{
			synchronized(messages)
			{
				SimpleMessage pa=getMessage(parent);
				i=pa.children.indexOf(msgid);
				pa.children.remove(msgid);
				messages.remove(msgid);
			}
		}
		else
		{	synchronized(messages)
			{
				messages.remove(msgid);
				i=-1;
			}
		}
		
		fireDeleteMessage(parent,i,msgid);
		
		
	}
	
	public Long getParentMessageId(Long id)
	{
		synchronized(messages)
		{
			return getMessage(id).parent;
		}
	}
	
	public SimpleMessage getMessage(Long id)
	{
		synchronized(messages)
		{
			return (SimpleMessage)messages.get(id);
		}
	}
	
	protected Vector listeners=new Vector();
	
	public void addMessageBoxListener(MessageBoxListener messageboxlistener)
	{
		listeners.add(messageboxlistener);
	}
	
	public void removeMessageSetListener(MessageBoxListener messageboxlistener)
	{
		listeners.remove(messageboxlistener);
	}
	
	private void fireNewMessage(Long msgid)
	{
		Iterator i=listeners.iterator();
		
		while(i.hasNext())
		{
			MessageBoxListener msl=(MessageBoxListener)i.next();
			msl.newMessage(msgid);
		}
	}
	
	private void fireDeleteMessage(Long parent,int index,Long msgid)
	{
		Iterator i=listeners.iterator();
		
		while(i.hasNext())
		{
			MessageBoxListener msl=(MessageBoxListener)i.next();
			msl.deleteMessage(parent,index,msgid);
		}
	}
}

