/**
 * MessageSetTreeModel.java
 *
 * A dynamic, "rootlesss" tree model
 *
 * @author Dj Walker-Morgan
 */


import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import javax.swing.SwingUtilities;
import javax.swing.tree.TreePath;
import java.lang.reflect.InvocationTargetException;

public class MessageBoxTreeModel extends AbstractTreeModel implements MessageBoxListener
{
	
	MessageBox messagebox;
	
	public MessageBoxTreeModel(MessageBox ms)
	{
		super();
		this.messagebox=ms;
		this.messagebox.addMessageBoxListener(this);
	}
	
	class RootObject implements Serializable{};
	
	RootObject root=new RootObject();
	
	ArrayList roots=new ArrayList();
	
	public Object getRoot()
	{
		return root;
	}
	
	public Object getChild(Object parent, int index)
	{
		if(parent instanceof RootObject)
		{
			return roots.get(index);
		}
		
		return messagebox.getChildMessageId((Long)parent,index);
	}
	
	public  int getChildCount(Object parent)
	{
		if(parent instanceof RootObject)
		{
			return roots.size();
		}
		
		return messagebox.getNumberOfCommentingMessages((Long)parent);
	}
	
	public  boolean isLeaf(Object node)
	{
		if(node instanceof RootObject) return false;
		
		return getChildCount(node)==0;
	}
	
	public void valueForPathChanged(TreePath path, Object newValue)
	{
		// TODO
	}
	
	
	public void newMessage(final Long msgid)
	{
		
		Long parent=messagebox.getParentMessageId(msgid);
		TreePath path=null;
		int index=0;
		
		if(parent==null)
		{
			roots.add(msgid);
			path=getTreePath(root);
			index=roots.indexOf(msgid);
		}
		else
		{
			index=getIndexOfChild(parent,msgid);
			path=getTreePath(parent);
		}
		
		final TreePath activepath=path;
		final int activeindex=index;
		
		SwingUtilities.invokeLater(new Runnable()
								   {
					public void run()
					{
						fireChildAdded(activepath,activeindex,msgid);
					}});
	}
	
	/**
	 * Method updatedMessage
	 *
	 * @param    msgid               a  Long
	 *
	 */
	public void updatedMessage(Long msgid)
	{
		firePathChanged(getTreePath(msgid));
	}
	
	/**
	 * Method deleteMessage
	 *
	 * @param    msgid               a  Long
	 *
	 */
	public void deleteMessage(Long parentid,int i,final Long msgid)
	{
		int index=i;
		
		TreePath path=null;
		
		if(parentid==null)
		{
			index=roots.indexOf(msgid);
			roots.remove(index);
			path=getTreePath(root);
		}
		else
		{
			path=getTreePath(parentid);
			index=i;
		}
		
		final TreePath activepath=path;
		final int activeindex=index;
		
		SwingUtilities.invokeLater(new Runnable()
								   {
					public void run()
					{
						fireChildRemoved(activepath,activeindex,msgid);
					}});
		
	}
	
	
	
	
	
	public TreePath getTreePath(Object id)
	{
		ArrayList al=new ArrayList();
		
		if(id instanceof RootObject)
		{
			return new TreePath(root);
		}
		
		Long curr=(Long)id;
		
		while(curr!=null)
		{
			al.add(curr);
			curr=messagebox.getParentMessageId(curr);
		}
		
		al.add(root);
		
		Collections.reverse(al);
		
		return new TreePath(al.toArray());
	}
}

